import { useContext } from "react";
import moment from "moment";
import { v4 as uuidv4 } from "uuid";
import JobContext from "@qltr/jobctx";
import Initiator from "@qltr/initiator";
import { jobStatus, socketUrl } from "./job-config.js";

export function usePipelineCore() {
  const { state, jobUpdate, jobCreate, jobDelete, updatePipelines } =
    useContext(JobContext);

  function pipelineJob(pl, plReq) {
    const initiator = new Initiator(socketUrl);
    const jobId = uuidv4();
    const job = {
      status: jobStatus.PENDING,
      jobId,
      isPipeline: true,
      initiator,
      pipelineId: pl.id,
      branchId: plReq.pipeline.__test,
    };

    jobCreate(job);
    const onLog = (d) => {
      const job = state.jobs.find((j) => j.jobId === jobId);
      job.log.push(d);
      job.status = jobStatus.ACTIVE;
      jobUpdate({ ...job }, jobId);
    };

    const onClose = (c) => {
      const job = state.jobs.find((j) => j.jobId === jobId);
      job.exitcode = c;
      job.status = c === 0 ? jobStatus.OK : jobStatus.ERROR;
      jobUpdate({ ...job }, jobId);
    };

    const onPipelineTrigger = applyOnPipelineTrigger(pl, plReq);
    const started = initiator.newPipelineJob(
      plReq,
      onLog,
      onClose,
      () => {},
      onPipelineTrigger
    );
    started.then(() => jobUpdate({ status: jobStatus.ACTIVE }, jobId));
  }

  function pipelineStart(builderCache) {
    const { tree, branches, selectedBranches, triageFailing } = builderCache;
    const __test = Object.keys(tree)[0];
    const plReq = {
      pipeline: { __test, triggers: { ...tree[__test] } },
      isTriage: triageFailing,
    };
    const id = uuidv4();
    const pipeline = { id, branches, pendingTriggers: [], selectedBranches };
    const { pipelines } = state;
    pipelines.push(pipeline);
    updatePipelines([...pipelines]);
    pipelineJob(pipeline, plReq);
    return pipeline;
  }

  function pipelineCancel(pipelineId) {
    const pipeline = state.pipelines.find((p) => p.id === pipelineId);
    pipeline.isCanceled = true;
    pipeline.pendingTriggers.forEach((t) => clearTimeout(t));
    const jobs = state.jobs.filter(
      (j) => j.isPipeline && j.pipelineId === pipelineId
    );
    for (var j of jobs) {
      if (j.initiator.sk) j.initiator.sk.close();
      j.status = jobStatus.CANCELED;
      jobUpdate({ ...j }, j.jobId);
    }
  }

  function pipelineDestroy(pipelineId) {
    const pipelineIndex = state.pipelines.findIndex((p) => p.id === pipelineId);
    const pipeline = state.pipelines[pipelineIndex];
    pipeline.pendingTriggers.forEach((t) => clearTimeout(t));
    const jobs = state.jobs.filter(
      (j) => j.isPipeline && j.pipelineId === pipelineId
    );
    var isTriggered, isCompleted, isCanceled;
    for (var j of jobs) {
      isTriggered = !!j.initiator.sk;
      isCompleted = status === jobStatus.OK || status === jobStatus.ERROR;
      isCanceled = status === jobStatus.CANCELED;
      if (isTriggered && !isCompleted && !isCanceled) j.initiator.sk.close();
      jobDelete(j.jobId);
    }
    state.pipelines.splice(pipelineIndex, 1);
  }

  function applyOnPipelineTrigger(pl, plReq) {
    return function onPipelineTrigger(p) {
      const { triggers } = p;
      for (const t in triggers) {
        if (t === "__testDelay") continue;
        const delay = triggers[t].__testDelay ?? 0;
        delete triggers[t].__testDelay;
        const plTrigger = { ...p, triggers: triggers[t], __test: t };
        const jobReq = { ...plReq, pipeline: plTrigger };
        const triggerQueued = moment();
        const triggerAt = moment().add(delay, "ms");
        const testName = t;
        function removeTrigger() {
          const i = pl.pendingTriggers.findIndex((pt) => pt.testName === t);
          if (i < 0) return;
          pl.pendingTriggers.splice(i, 1);
        }
        function launchTrigger() {
          pipelineJob(pl, jobReq);
          removeTrigger();
        }
        const timer = setTimeout(launchTrigger, delay);
        const triggerTimings = { timer, triggerQueued, triggerAt };
        const doTrigger = { removeTrigger, launchTrigger, jobReq };
        pl.pendingTriggers.push({ testName, ...triggerTimings, ...doTrigger });
      }
    };
  }

  return { pipelineStart, pipelineCancel, pipelineDestroy };
}
